/* Dump out the cn and automountInformation values for the auto.master map,
   regardless of the character set used, so that we can parse them from the
   shell script that starts autofs.  Maybe it'll be useful for debugging. */

#include <ldap.h>
#include <locale.h>
#include <stdio.h>
#include <string.h>

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define KEY "cn"
#define VALUE "automountInformation"
#define MAPOC "automountMap"
#define ENTRYOC "automount"

/* We don't free anything here, but then we're not around for long, either. */

int
main(int argc, const char **argv)
{
	LDAP *ld = NULL;
	LDAPControl *server = NULL, *client = NULL;
	LDAPMessage *messages = NULL, *entry = NULL;
	int result;
	int version = LDAP_VERSION3;
	char *attrs[] = {KEY, VALUE, NULL}, *dn = "";

	setlocale(LC_ALL, "");

	ld = ldap_init(NULL, LDAP_PORT);
	if(ld == NULL) {
		fprintf(stderr, "%s: error initializing LDAP\n", argv[0]);
		return 1;
	}

	result = ldap_simple_bind_s(ld, NULL, NULL);
	if(result != LDAP_SUCCESS) {
		fprintf(stderr, "%s: error binding to server: %s\n",
			argv[0], ldap_err2string(result));
		ldap_unbind(ld);
		return 2;
	}

	result = ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &version);
	if(result != LDAP_SUCCESS) {
		fprintf(stderr, "%s: error switching to LDAPv3 on server: %s\n",
			argv[0], ldap_err2string(result));
		ldap_unbind(ld);
		return 3;
	}

	result = ldap_search_ext_s(ld, NULL,
				   LDAP_SCOPE_SUBTREE,
				   "(&(objectclass=" MAPOC ")"
				     "(ou=auto.master))",
				   attrs, FALSE,
				   &server, &client,
				   NULL,
				   LDAP_NO_LIMIT,
				   &messages);
	if(result != LDAP_SUCCESS) {
		fprintf(stderr, "%s: error searching for auto.master map: %s\n",
			argv[0], ldap_err2string(result));
		ldap_unbind(ld);
		return 4;
	}

	entry = ldap_first_entry(ld, messages);
	if(entry == NULL) {
		fprintf(stderr, "%s: no auto.master map found: %s\n",
			argv[0], ldap_err2string(result));
		ldap_unbind(ld);
		return 5;
	}

	dn = ldap_get_dn(ld, entry);
	if((dn == NULL) || (strlen(dn) == 0)) {
		fprintf(stderr, "%s: auto.master map has no dn: %s\n",
			argv[0], ldap_err2string(result));
		ldap_unbind(ld);
		return 6;
	}

	result = ldap_search_ext_s(ld, dn,
				   LDAP_SCOPE_SUBTREE,
				   "(objectclass=" ENTRYOC ")",
				   attrs, FALSE,
				   &server, &client,
				   NULL,
				   LDAP_NO_LIMIT,
				   &messages);
	if(result != LDAP_SUCCESS) {
		fprintf(stderr, "%s: error searching auto.master: %s\n",
			argv[0], ldap_err2string(result));
		ldap_unbind(ld);
		return 7;
	}

	for(entry = ldap_first_entry(ld, messages);
	    entry != NULL;
	    entry = ldap_next_entry(ld, entry)) {
		char **cn, **values;
		cn = ldap_get_values(ld, entry, KEY);
		values = ldap_get_values(ld, entry, VALUE);
		if(cn && cn[0] && values && values[0]) {
			printf("%s %s\n", cn[0], values[0]);
		}
	}

	ldap_unbind(ld);
	return 0;
}
